// Copyright (c) 2019 Samsung Research America
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef NAV2_CORE__RECOVERY_HPP_
#define NAV2_CORE__RECOVERY_HPP_

#include <string>
#include <memory>

#include "rclcpp/rclcpp.hpp"
#include "nav2_util/lifecycle_node.hpp"
#include "tf2_ros/buffer.h"
#include "nav2_costmap_2d/costmap_topic_collision_checker.hpp"

namespace nav2_core
{

/**
 * @class Recovery
 * @brief Abstract interface for recoveries to adhere to with pluginlib
 */
class Recovery
{
public:
  using Ptr = std::shared_ptr<Recovery>;

  /**
   * @brief Virtual destructor
   */
  virtual ~Recovery() {}

  /**
   * @param  parent pointer to user's node
   * @param  name The name of this planner
   * @param  tf A pointer to a TF buffer
   * @param  costmap_ros A pointer to the costmap
   */
  virtual void configure(
    const rclcpp_lifecycle::LifecycleNode::SharedPtr parent,
    const std::string & name, std::shared_ptr<tf2_ros::Buffer> tf,
    std::shared_ptr<nav2_costmap_2d::CostmapTopicCollisionChecker> collision_checker) = 0;

  /**
   * @brief Method to cleanup resources used on shutdown.
   */
  virtual void cleanup() = 0;

  /**
   * @brief Method to active recovery and any threads involved in execution.
   */
  virtual void activate() = 0;

  /**
   * @brief Method to deactive recovery and any threads involved in execution.
   */
  virtual void deactivate() = 0;

  /**
   * @brief Method Execute recovery behavior
   * @param  name The name of this planner
   * @return true if successful, false is failed to execute fully
   */
  // TODO(stevemacenski) evaluate design for recoveries to not host
  // their own servers and utilize a recovery server exposed action.
  // virtual bool executeRecovery() = 0;
};

}  // namespace nav2_core

#endif  // NAV2_CORE__RECOVERY_HPP_
